C***********************************************************************
C*                     D U B N A                                       *
C*          JOINT INSTITUTE FOR NUCLEAR RESEARCH                       *
C*         LABORATORY OF INFORMATION TECHNOLOGIES                      *
C*         PUZYNIN I.V.,PUZYNINA T.P., STRIZH T.A.                     *
C*                      SLIPH4.F                                       *
C*         REPORT JINR, P-11-87-332, DUBNA, 1987                       *
C*                                                                     *
C*   THIS PROGRAM COMPLEX IS INTENDING FOR SOLVING OF STURM-           *
C*         LIOUVILLE'S PROBLEM FOR DIFFERENTIAL EQUATION               *
C*          D2Y/DX2+2*P(X)*DY/DX+(Q(X)-EV*R(X))*Y=0                    *
C*             WITH  BOUNDARY  CONDITIONS                              *
C*               D1*DY/DX+F1*Y=0  AT X=A                               *
C*               D2*DY/DX+F2*Y=0  AT X=B                               *
C*       USING CONTINUE ANALOG OF NEWTON'S METHOD                      *
C*        DISCRETE SCHEMES WITH PRECISION O(H**4)                      *
C*                                                                     *
C* TEST:  SOLUTION OF LEGENDRE'S EQUATION                              *
C* TEST:  POTENTIAL HAS FORM P(X)=-X/(1-X*X),Q(X)=0,R(X)=1/(1-X*X)     *                                          *
C* TEST:  BOUNDARY CONDITIONS D1=1  DD1=0                              *
C* TEST:  BOUNDARY CONDITIONS F1=0  DF1=0 FOR NZ=0                     *
C* TEST:  BOUNDARY CONDITIONS F1=-0.5*EV  DF1=-0.5 FOR NZ.GT.0         *
C* TEST:  BOUNDARY CONDITIONS D2=1  DD2=0                              *
C* TEST:  BOUNDARY CONDITIONS F2=0 DF2=0 FOR NZ=0                      *
C* TEST:  BOUNDARY CONDITIONS F2=0.5*EV  DF2=0.5 FOR NZ.GT.0           *
C* TEST:  A=-1  B=1  H=2/(NDIM-1)                                      *
C*                                                                     *
C* TEST: ANALYTICAL EV_NZ=-NZ*(NZ+1), NZ-NUMBER OF EIGEN FUNCTION ZEROS*
C***********************************************************************

      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NDIM=6401)
      DIMENSION X(NDIM),Y0(NDIM),Y(NDIM),P(NDIM),Q(NDIM),R(NDIM),
     ,S(NDIM),AN(NDIM),BN1(NDIM),V1(NDIM)
      DIMENSION EVA(20),EVANALYT(20)
      COMMON /BOUND/ D1,DD1,F1,DF1,D2,DD2,F2,DF2
      COMMON /HVAR/ H,HH,H1,H2,H3,H26,H212
      COMMON /CMINMA/ CMIN,CMAX
      COMMON /MOD/ AK,BK,MOD
      COMMON /SWITCH/ LINFUN,LOUT,LQ,LEVZ
      COMMON /NZERO/ NIS

      DO I=1,11
      NZ=I-1
      EVANALYT(I)=-NZ*(NZ+1)
      END DO
COMMENT: NZ IS NUMBER OF ZEROS OF EIGEN FUNCTION

C OUTPUT FILE SLIPH4_1.DAT FOR MOD=2 CONTAINTS ALL EIGEN FUNCTIONS
C OUTPUT FILE SLIPH4_2.DAT CONTAINTS ALL NECESSARY INFORMATION
C OBTAINED IN PROCESS OF CALCULATIONS
      OPEN(1,FILE='SLIPH4_1.DAT')
      OPEN(2,FILE='SLIPH4_2.DAT')
C
      WRITE(2,*)' TEST SLIPH4:  P(X)=-X/(1-X*X), Q(X)=0, R(X)=1/(1-X*X)'
      WRITE(2,*)' TEST SLIPH4: BOUNDARY CONDITIONS D1=1 DD1=0'
      WRITE(2,*)' TEST SLIPH4: BOUNDARY CONDITIONS F1=0 DF1=0 FOR NZ=0'
      WRITE(2,*)' TEST SLIPH4: BOUNDARY CONDITIONS F1=-0.5*EV DF1=-0.5',
     ,' FOR NZ.GT.0'
      WRITE(2,*)' TEST SLIPH4: BOUNDARY CONDITIONS D2=1 DD2=0'
      WRITE(2,*)' TEST SLIPH4: BOUNDARY CONDITIONS F2=0 DF2=0 FOR NZ=0'
      WRITE(2,*)' TEST SLIPH4: BOUNDARY CONDITIONS F2=0.5*EV DF2=.5',
     ,' FOR NZ.GT.0'
      WRITE(2,*)' TEST SLIPH4: ANALYT EV_NZ=-NZ*(NZ+1), WERE NZ IS ',
     ,' NUMBER OF ZEROS OF EIGEN FUNCTION'
      WRITE(2,*)'*****************************************************'

      PRINT 1002
      WRITE(2,1002)
      CMIN=1.E-30
      CMAX=1.E+30
      NMAX=NDIM
      NEVMAX=20

C  PARAMETERS FOR DISCRETE SCHEME
      A=-1.D0
      B=1.D0
      N=NDIM
      H=(B-A)/(N-1)
      HH=H*H
      H1=1.D0/H
      H2=H1*H1
      H3=H1*H2
      H26=HH/6.D0
      H212=0.5D0*H26

C   PARAMETERS FOR SLIPH4
C*
C*  LAS=0 ASYMPTOTICA IS ABSENT. IF LAS.NE.0 SUBROUTINE ASIM MUST TO BE
C*  LINFUN=0  USER GIVES INITIAL EIGEN FUNCTION
C*  LOUT=0 CHECK OF EIGEN VALUES OUTSIDE OF INTERVAL (XMIN+DX,XMAX-DX)
C*  LQ=0 USER GIVS VALUES XMIN,XMAX,XW,M IN SUBROUTINE PQRX
C*  LEVZ=0 TRANSITION TO END OF CYCLE 'DO 11' WITH DIAGNOSTIC:
C* EIGEN VALUE0 IS NOT FOUND IN SUBROUTINE NEWTON
C*
      LAS=0
      LINFUN=1
      LOUT=0
      LQ=1
      LEVZ=1
C*
C FOR MOD=0 -  THE ORDINARY NEWTON'S METHOD (P.2.2)
C FOR MOD=2 -  THE MODIFIED NEWTON'S METHOD (P.2.3)
      MOD=0

C PRECISION OF CALCULATION: EXIT FROM SLIPH4 WHEN RESIDUAL.LE.EPS
      EPS=1.D-5

C MAXIMAL NUMBER OF ITERATIOS
      NT=100

C FOR CHOICE OF FORMULA FOR CALCULATION OF PARAMETER TAU
      LST=3

C STEP FOR PRINT OF ITERATIONS IN SLIPH4
      LPR=1

C STEP BY X FOR PRINT OF EIGEN FUNCTION AND SYSTEM COEFFICIETS
      LX=N/10

C INITIAL VALUE OF ITERATION PARAMETER TAU
      T0=0.1D0

C FOR LDEL=1 RESIDUAL IS IN NORMA C
C FOR LDEL=2 RESIDUAL IS MEAN SQUARE
      LDEL=1

      WRITE(*,*)' A=',A,' B=',B,' N=',N,' H=',H,' EPS=',EPS,' NT=',NT
      WRITE(2,*)' A=',A,' B=',B,' N=',N,' H=',H,' EPS=',EPS,' NT=',NT
      WRITE(*,*)' LST=',LST,' LPR=',LPR,' LX=',LX,' T0=',T0
      WRITE(2,*)' LST=',LST,' LPR=',LPR,' LX=',LX,' T0=',T0
      WRITE(*,*)' LDEL=',LDEL,' MOD =',MOD,' LQ=',LQ
      WRITE(2,*)' LDEL=',LDEL,' MOD =',MOD,' LQ=',LQ

      CALL PQRX(A,B,N,P,Q,R,X,XMIN,XMAX,XW,M)
      PRINT 1017
      WRITE(2,1017)
      PRINT 1018,(X(J),P(J),Q(J),R(J),J=1,N,LX)
      WRITE(2,1018)(X(J),P(J),Q(J),R(J),J=1,N,LX)
      N1=N-1
      IF(LQ.EQ.0) GO TO 1
      CALL MINMAX(N,X,Q,XMIN,XMAX,XW,M)
    1 DX=H*H
      EPSX=H*H
      ITMIN=3
      ITMAX=20
      LSTEP=1
      PRINT 1025,XMIN,XMAX,XW,M
      WRITE(2,1025)XMIN,XMAX,XW,M
      PRINT 1032
      CONST=1.1D0
      EV0=XMAX
      WRITE(2,*)' *****************************************************'

C PARAMETERS FOR CYCLE NUMBER OF SOLUTIONS
      NNECS=1
      NNECF=3
      DO 11 NNEC=NNECS,NNECF
      NIS=NNEC-1
      CALL D1F1(EV0,A,D1,DD1,F1,DF1)
      CALL D2F2(EV0,B,D2,DD2,F2,DF2)

C INITIAL APPROXIMATION EV0=EVA(NNEC),Y0
      CALL NEWTON(A,B,P,Q,R,N,XMIN,XMAX,XW,M,EVA,X,Y0,DX,EPSX,
     ,ITMIN,ITMAX,LSTEP,MMAX,NIS,NNEC,NEVMAX)

      EV0=EVA(NNEC)
      IF(MMAX.NE.0) GO TO 2
      PRINT 1000
      WRITE(2,1000)
      IF(LEVZ.EQ.0) GO TO 10
    2 CALL ZEROS(N,Y0,NZ)
      WRITE(*,*)' NZ=',NZ,' EV0=',EV0
      WRITE(2,*)' NZ=',NZ,' EV0=',EV0

      CALL SLIPH4(A,B,N,NMAX,P,Q,R,EV0,Y0,LAS,Y,EPS,NT,LST,LPR,LX,T0,X,
     ,LDEL,AN,BN1,V1,S)

      DO 3 I=1,N
    3 Y(I)=Y0(I)
      EV=EV0
      CALL ZEROS(N,Y,NZ)
      WRITE(*,*)' NZ=',NZ,' EV_ANALYT=',EVANALYT(NZ+1),' EV=',EV
      WRITE(2,*)' NZ=',NZ,' EV_ANALYT=',EVANALYT(NZ+1),' EV=',EV
      PRINT 1031,NZ
      WRITE(2,1031)NZ
      IF(MOD.EQ.0) CALL UNORM(N,H,Y)
      IF(MOD.EQ.2) GO TO 20
      PRINT 2000
      WRITE(2,2000)
      GO TO 21
   20 EVA(NNEC)=EV0

C ORTOGONALIZATION OF EIGEN FUNCTION
      CALL DOPORT(NIS,N,H,Y,AN,BN1,1)

      PRINT 1033
      WRITE(2,1033)
   21 PRINT 1034,(J,X(J),Y(J),J=1,N,LX)
      WRITE(2,1034)(J,X(J),Y(J),J=1,N,LX)
   12 PRINT 1032
      WRITE(2,1032)
   10 CONTINUE
      WRITE(2,*)' *****************************************************'
   11 CONTINUE
      PRINT 1003
      WRITE(2,1003)
      CLOSE (1)
      CLOSE (2)

 1000 FORMAT(/,3X,'EIGEN VALUE0 IS NOT FOUND IN NEWTON')
 1002 FORMAT(1X,17('*'),' BEGIN TEST SLIPH4 ',17('*'))
 1003 FORMAT(1X,19('*'),' END TEST SLIPH4 ',19('*'))
 1017 FORMAT(7X,'X',11X,'P',11X,'Q',11X,'R')
 1018 FORMAT(2X,4D12.3)
 1025 FORMAT(3X,'XMIN=',D16.9,' XMAX=',D16.9,' XW=',D16.9,' M=',I5)
 1031 FORMAT(1X,'NUMBER OF ZEROS OF FUNCTION Y  NZ=',I4)
 1032 FORMAT(1X,55('-'),/)
 1033 FORMAT(5X,'J',8X,'X',9X,'YORT')
 1034 FORMAT(1X,I6,D12.4,D17.9)
 2000 FORMAT(4X,'J',8X,'X',12X,'Y')
      END

C***********************************************************************

      SUBROUTINE PQRX(A,B,N,P,Q,R,X,XMIN,XMAX,XW,M)
      IMPLICIT REAL*8(A-H,O-Z)
C*  SUBROUTINE FOR CALCULATION ARRAYS  P,Q,R,X

      COMMON /HVAR/ H,HH,H1,H2,H3,H26,H212
      COMMON /SWITCH/ LINFUN,LOUT,LQ,LEVZ
      DIMENSION P(N),Q(N),R(N),X(N)

      X(1)=A
      X(N)=B
      N1=N-1
      DO 1 I=2,N1
      X(I)=A+H*(I-1)
      X2M1=1.D0-X(I)*X(I)
      P(I)=-X(I)/X2M1
      Q(I)=0.D0
      R(I)=1.D0/X2M1
    1 CONTINUE
C*  IF USER GIVS HERE VALUES XMIN,XMAX,XW,M THEN HE MUST PUT LQ=0

      RETURN
      END

C***********************************************************************

      SUBROUTINE D1F1(EV,A,D1,DD1,F1,DF1)
      IMPLICIT REAL*8(A-H,O-Z)
C*
C*  SUBROUTINE FOR CALCULATION COEFFICIENTS D1,F1 IN  LEFT BOUNDARY
C*  CONDITIONS AND HIS DERIVATIVES DD1,DF1

      COMMON /SWITCH/LINFUN,LOUT,LQ,LEVZ
      COMMON/NZERO/NZ

C*  TEST SLIPH4: FOR NZ=0 D1=1,F1=0
C*  TEST SLIPH4: FOR NZ.GT.0 D1=1,F1=-EV/2
C*  TEST SLIPH4:  D2=1,F2=EV/2,DF2=1/2

      D1=1.D0
      DD1=0.D0
      F1=0.D0
      DF1=0.D0
      IF(NZ.EQ.0) RETURN
      F1=-0.5D0*EV
      DF1=-0.5D0
      RETURN
      END

C***********************************************************************

      SUBROUTINE D2F2(EV,B,D2,DD2,F2,DF2)
      IMPLICIT REAL*8(A-H,O-Z)
C*
C*  SUBROUTINE FOR CALCULATION COEFFICIENTS D2,F2 IN RIGHT BOUNDARY
C*  CONDITIONS AND HIS DERIVATIVES DD2,DF2

      COMMON /SWITCH/LINFUN,LOUT,LQ,LEVZ
      COMMON/NZERO/NZ

      D2=1.D0
      DD2=0.D0
      F2=0.D0
      DF2=0.D0
      IF(NZ.EQ.0) RETURN
      F2=0.5D0*EV
      DF2=0.5D0
      RETURN
      END

C***********************************************************************

      SUBROUTINE UNORM(N,H,Y)
      IMPLICIT REAL*8(A-H,O-Z)
C*
C*  FUNCTION Y IS DEVIDED ON INTEGRAL FROM Y*Y

      DIMENSION Y(N)

      S=Y(1)*Y(1)+Y(N)*Y(N)+4.D0*Y(2)*Y(2)
      N2=N-2
      DO 1 I=3,N2,2
      S=S+2.D0*Y(I)*Y(I)+4.D0*Y(I+1)*Y(I+1)
    1 CONTINUE
      S=DSQRT(S*H/3.D0)
      DO 2 J=1,N
      Y(J)=Y(J)/S
    2 CONTINUE
      RETURN
      END

C***********************************************************************

      SUBROUTINE ZEROS(N,Y,NZ)
      IMPLICIT REAL*8(A-H,O-Z)
C*
C*  NUMBER OF ZEROS OF FUNCTION Y IS CALCULATED

      DIMENSION Y(N)

      N2=N-2
      NZ=0
      DO 1 I=2,N2
      IF(Y(I)*Y(I+1).GT.0.D0) GO TO 1
      NZ=NZ+1
      IF(Y(I+1).EQ.0..AND.I.NE.N2) NZ=NZ-1
    1 CONTINUE
C*  NUMBER OF ZEROS EQUALS NUMBER OF CHANGES OF SIGN OF FUNCTION Y
      RETURN
      END

C**********************************************************************

      SUBROUTINE MINMAX(N,X,Q,XMIN,XMAX,XW,M)
      IMPLICIT REAL*8(A-H,O-Z)
C*
C*   FOR LQ.NE.0 ARE CALCULATED XMIN=QMIN, XMAX=QMAX AND POINT
C*   OF SEW TOGETHER  XW  AND  HER  NUMBER  M

      DIMENSION Q(N),X(N)
      COMMON /CMINMA/ CMIN,CMAX
      COMMON /SWITCH/ LINFUN,LOUT,LQ,LEVZ

      IF(LQ.EQ.0) GO TO 4
      XMIN=CMAX
      XMAX=-CMAX
      N1=N-1
      DO 2 I=2,N1
      IF(Q(I).LE.XMAX) GO TO 1
      XMAX=Q(I)
      XW=X(I)
      M=I
    1 IF(Q(I).LT.XMIN) XMIN=Q(I)
    2 CONTINUE
    4 MM=M
      XM=XW
      IF(M.GT.3) GO TO 3
      M=4
      XW=X(M)
    5 PRINT 10,MM,XM,M,XW
      RETURN
   3  N2=N-2
      IF(M.LT.N2) RETURN
      M=N-3
      XW=X(M)
      GO TO 5
   10 FORMAT(3X,'XW OLD(',I6,')=',D16.9,' XW NEW(',I6,')=',D16.9)
      END

C***********************************************************************

      SUBROUTINE NEWTON(A,B,P,Q,R,N,XMIN,XMAX,XW,M,EV,X,Y,DX,EPS,ITMIN,
     ,ITMAX,LSTEP,MMAX,NIS,NNEC,NEVMAX)
      IMPLICIT REAL*8(A-H,O-Z)
C*  SUBROUTINE FOR FINDING OF NODES FUNCTION T WITH USING ALGORITHM
C*  JINR COMMUNICATION 5-10-992,1977,DUBNA.

      COMMON /CMINMA/ CMIN,CMAX
      COMMON /SWITCH/ LINFUN,LOUT,LQ,LEVZ
      DIMENSION X(N),Y(N),P(N),Q(N),R(N)
      DIMENSION EV(NEVMAX)

      MMAX=0
      N1=NIS+1
      EV(N1)=XMAX
      IF(LSTEP.EQ.0) EV(N1)=XMIN
      IF(NIS.NE.0) EV(N1)=EV(NIS)
      N11=N1+1
      DO 7  J=N1,NNEC
      IF(J.EQ.N11) EV(J)= EV(J-1)-DX*(-1.D0)**LSTEP
      IF(J.GT.N11) EV(J)=0.5D0*(EV(J-1)+EV(J-2))
      EV(J)=EV(J)-DX*(-1.D0)**LSTEP
      DO 2 I=1,ITMAX
      XX=EV(J)
  10  CALL TTP(A,B,P,Q,R,XX,T,TP,M,N,Y)
      ZNA=TP-T*SIGMA(XX,EV,J,NEVMAX)
      IF(DABS(ZNA).LE.CMIN) GO TO 9
      EV(J)=XX-T/ZNA
      IF(LOUT.EQ.0) GO TO 8
      IF(EV(J).LT.(XMIN-DX)) GO TO 5
      IF(EV(J).GT.(XMAX+DX)) GO TO 6
    8 ALAM=EV(J)
      IF(I.LE.ITMIN) GO TO 2
    1 IF(DABS(EV(J)-XX).LE.EPS.AND.DABS(T).LE.EPS) GO TO 4
      GO TO 2
   9  XWS=XW
      MS=M
      M=M+N/100
      XW=X(M)
      PRINT 123,CMIN,MS,XWS,M,XW
      GO TO 10
    2 CONTINUE
      DXX=XX-EV(J)
      PRINT 20,EPS,ITMAX,XX,J,EV(J),DXX,T
      GO TO 7
    4 MMAX=MMAX+1
      PRINT 23,I
   23 FORMAT(3X,'FROM NEWTON: NITER=',I3)
    7 CONTINUE
      RETURN
   20 FORMAT(/,1X,'FROM NEWTON: NODE IS ABSENT WITH GIVEN EPS=',D10.3,
     ,'AFTER ITER=',I3,/,
     ,' XX=',D10.4,' EV0(',I3,')=',D10.4,'DXX=',D10.4,' T=',D10.4)
    3 CONTINUE
      MMAX=1
      RETURN
    5 PRINT 21,EV(J),XMIN
      RETURN
  21  FORMAT(1X,'FROM NEWTON EV0=',D16.9,' LESS THEN XMIN=',D16.9)
    6 PRINT 22,EV(J),XMAX
      RETURN
  123 FORMAT(1X,'FROM NEWTON: ZNA.LT.CMIN=',D10.3,' M_OLD=',I4,
     ,' XW_OLD=',D12.5,' M_NEW=',I4,' XW_NEW=',D12.5)
  22  FORMAT(1X,'FROM NEWTON: EV0=',D16.9,' GRATER THEN X_MAX=',D16.9)
      END

C***********************************************************************

      FUNCTION SIGMA(XX,X,I,IMAX)
      IMPLICIT REAL*8(A-H,O-Z)
C*  FUNCTION SIGMA IS NEEDED FOR FINDING ZEROS POLYNOM T
C*  WITH HELP NEWTON METHOD

      DIMENSION X(IMAX)

      SIGMA=0.D0
      I1=I-1
      IF(I1.LT.1)RETURN
      DO 1 J=1,I1
    1 SIGMA=SIGMA+1.D0/(XX-X(J))
      RETURN
      END

C***********************************************************************

      SUBROUTINE TTP(A,B,P,Q,R,EV,T,TP,M,N,Y)
      IMPLICIT REAL*8(A-H,O-Z)
C*  TTP IS USED FOR CALCULATION POLINOM T VALUE AND HIS DERIVATIVE TP

      DIMENSION Y(N),P(N),Q(N),R(N)
      COMMON / CMINMA/CMIN,CMAX
      COMMON / HVAR/ H,HH,H1,H2,H3,H26,H212

      CALL D1F1(EV,A,D1,DD1,F1,DF1)
      CALL D2F2(EV,B,D2,DD2,F2,DF2)
      N1=N-1
      N3=N-3
      N3M=N3+M
      S11=-3.D0*D1+2.D0*H*F1
      S11P=-3.D0*DD1+2.D0*H*DF1
      S21=4.D0*D1
      S21P=4.D0*DD1
      S31=-D1
      S31P=-DD1
      Y(2)=H
      Y(N-1)=H
      Y2P=0.D0
      YNM1P=0.D0
      SM1I=1.D0-H*P(2)
      S0I=-2.D0+HH*(Q(2)-EV*R(2))
      S0IP=-HH*R(2)
      S1I=1.D0+H*P(2)
      A2=1.D0
      A2P=0.D0
      R1=S0I*S11-S21*SM1I
      R2=S1I*S11-S31*SM1I
      R22=R2*R2
      A3=-R1/R2
      A3P=((S21P*SM1I-S0I*S11P-S0IP*S11)*R2+R1*(S11P*S1I-S31P*SM1I))/R22
      Y(3)=A3*Y(2)
      Y3P=A3P*Y(2)
      Y(1)=-(S21*Y(2)+S31*Y(3))/S11
      DO 1 I=4,M
      SM1I=1.D0-H*P(I-1)
      S0I=-2.D0+HH*(Q(I-1)-EV*R(I-1))
      S0IP=-HH*R(I-1)
      S1I=1.D0+H*P(I-1)
      A4=-(SM1I*A2+S0I*A3)/S1I
      A4P=-(S0IP*A3+SM1I *A2P+S0I*A3P)/S1I
      Y(I)=A4*Y(2)
      Y4P=A4P*Y(2)
      IF(DABS(A4).LT.CMAX) GO TO 7
      A3=A3/A4
      A4P=A4P/A4
      A3P=A3P/A4
      Y3P=Y3P/A4
      Y4P=Y4P/A4
      A2=A2/A4
      A2P=A2P/A4
      Y2P=Y2P/A4
      DO 6 J=1,I
  6   Y(J)=Y(J)/A4
      A4=1.D0
      PRINT 20,I,CMAX
  20  FORMAT(1X,'FROM TTP:  FOR I=',I5,' A4.GT.CMAX=',E9.3,/
     ,1X,'IN LATER    A4=1.')
    7 CONTINUE
      IF(I.EQ.M) GO TO 1
      A2=A3
      A2P=A3P
      A3=A4
      A3P=A4P
      Y2P=Y3P
      Y3P=Y4P
    1 CONTINUE
      YML=Y(M)
      SM2N=D2
      SM2NP=DD2
      SM1N=-4.D0*D2
      SM1NP=-4.D0*DD2
      S0N=3.D0*D2+2.D0*H*F2
      S0NP=3.D0*DD2+2.D0*H*DF2
      SM1I=1.D0-H*P(N-1)
      S0I=-2.D0+HH*(Q(N-1)-EV*R(N-1))
      S0IP=-HH*R(N-1)
      S1I=1.D0+H*P(N-1)
      BNM1=1.D0
      BNM1P=0.D0
      RN1=S0I*S0N-SM1N*S1I
      RN2=SM2N*S1I-SM1I*S0N
      RN22=RN2*RN2
      BNM2=RN1/RN2
      Y(N-2)=BNM2*Y(N-1)
      Y(N)=-Y(N-1)*SM1N/S0N-Y(N-2)*SM2N/S0N
      BNM2P=((S0IP*S0N+S0I*S0NP-SM1NP*S1I)*RN2+RN1*(SM1I*S0NP-
     -SM2NP*S1I))/RN22
      YNM2P=BNM2P*Y(N-1)
      DO 2 J=M,N3
      I=N3M-J
      SM1I=1.D0-H*P(I+1)
      S0I=-2.D0+HH*(Q(I+1)-EV*R(I+1))
      S0IP=-HH*R(I+1)
      S1I=1.D0+H*P(I+1)
      BNM3=-(S0I*BNM2+S1I*BNM1)/SM1I
      BNM3P=-(S0IP*BNM2+S0I*BNM2P+S1I*BNM1P)/SM1I
      Y(I)=BNM3*Y(N-1)
      YNM3P=BNM3P*Y(N-1)
      IF(DABS(BNM3).LT.CMAX) GO TO 9
      BNM2=BNM2/BNM3
      BNM3P=BNM3P/BNM3
      BNM2P=BNM2P/BNM3
      YNM3P=YNM3P/BNM3
      YNM2P=YNM2P/BNM3
      BNM1=BNM1/BNM3
      BNM1P=BNM1P/BNM3
      YNM1P=YNM1P/BNM3
      DO 8 K=M,J
      IK=N3M-K
    8 Y(IK)=Y(IK)/BNM3
      BNM3=1.D0
      PRINT 21,I,CMAX
  21  FORMAT(1X,'FROM TTP:  FOR I=',I5,' BNM3.GT.CMAX=',E9.3,
     ,'IN LATER BNM3=1')
    9 CONTINUE
      IF(J.EQ.N3) GO TO 2
      BNM1=BNM2
      BNM2=BNM3
      BNM1P=BNM2P
      BNM2P=BNM3P
      YNM1P=YNM2P
      YNM2P=YNM3P
    2 CONTINUE
      T=Y(M)*(Y(M-2)-4.D0*Y(M-1))+6.D0*YML*Y(M)-YML*(4.D0*Y(M+1)-Y(M+2))
      TP=YNM3P*(Y(M-2)-4.D0*Y(M-1))+Y(M)*(Y2P-4.D0*Y3P)+6.D0*Y4P*Y(M)+
     +6.D0*YML*YNM3P-Y4P*(4.D0*Y(M+1)-Y(M+2))-YML*(4.D0*YNM2P-YNM1P)
      CY=YML/Y(M)
      DO 3 I=M,N
    3 Y(I)=CY*Y(I)
      SY=Y(1)*Y(1)-Y(N)*Y(N)
      DO 4 I=2,N1,2
      SY=SY+4.D0*Y(I)*Y(I)+2.D0*Y(I+1)*Y(I+1)
    4 CONTINUE
      ANOR=DSQRT(H*SY/3.D0)
      DO 5 I=1,N
    5 Y(I)=Y(I)/ANOR
      RETURN
      END

C***********************************************************************

      SUBROUTINE SLIPH4(A,B,N,NMAX,P,Q,R,EV0,Y0,LAS,Y,EPS,NT,LST,LPR,LX,
     ,T0,X,LDEL,AN,BN,V,S)
      IMPLICIT REAL*8(A-H,O-Z)

C--- SUBROUTINE FOR SOLVING OF STURME-LIOUVILLE'S PROBLEM
C--- WITH HELP OF DISCRETE NEWTON'S METHOD
C--- A AND B ARE LEFT AND RIGHT BOUNDARIES OF INTERVAL FOR VARIABLE X
C--- NZER - NUMBER OF EIGEN FUNCTION NODES ON INTERVAL (A,B)
C--- N - NUMBER OF POINTS OF DISCRETE SCHEME ON INTERVAL (A,B)
C--- ATTENSION!  N MUST BE ODD

      DIMENSION P(N),Q(N),R(N),Y0(N),Y(N),X(N),AN(N),BN(N),V(N),S(N)
      COMMON /ANBN/ A23,A24,A25,B2,A34,A35,B3
      COMMON /BOUND/ D1,DD1,F1,DF1,D2,DD2,F2,DF2
      COMMON /CI1/ AM11,AM21,AM31,AM41,AM51,AM61
      COMMON /CN/ AM4,AM3,AM2,AM1,AM0,AMP1
      COMMON /CN1/ C4N,C3N,C2N,C1N,C0N,CN1
      COMMON /CMINMA/ CMIN,CMAX
      COMMON /HVAR/ H,HH,H1,H2,H3,H26,H212
      COMMON /MOD/ AK,BK,MOD
      COMMON/NZERO/ NZER

      IF(N.LE.NMAX) GO TO 11
      PRINT 1000,N,NMAX
      RETURN

   11 N1=N-1
      H=(B-A)/N1
      N2=N-2
      HH=H*H
      H1=1.D0/H
      H2=H1*H1
      H3=H1*H2
      H26=HH/6.D0
      H212=0.5D0*H26
      CALL D1F1(EV0,A,D1,DD1,F1,DF1)
      CALL D2F2(EV0,B,D2,DD2,F2,DF2)
      DELT=0.D0
      DO 1 I=2,N1
      RD=DABS((Y0(I+1)-2.D0*Y0(I)+Y0(I-1))*H2+P(I)*(Y0(I+1)-Y0(I-1))*H1+
     +(Q(I)-EV0*R(I))*Y0(I))
      IF(DELT.GT.RD) GO TO 1
      DELT=RD
      XD=X(I)
    1 CONTINUE
      CALL DELT0(EV0,X,Y0,N,DELT,XD,P,Q,R)
      PRINT 1019,EV0,DELT,XD
      M=1
      TK=T0
      TK1=T0
      EV=EV0
      LRET=0
      EVF=EV0
C GO FROM MOD=2 TO MOD=0
      MODF=MOD
      DELK1=DELT
      DELK2=DELT
      DO 10 K=1,NT
      IF(MOD.EQ.0) GO TO 16
      DO 20 J2=2,N1
   20 S(J2)=0.
      CALL AKBK(A,B,N,H,EV0,Y0,1)
      GO TO 17
   16 CONTINUE
      DO 14 J=2,N1
   14 S(J)=R(J)*Y0(J)
   13 CALL AKBK(A,B,N,H,EV0,Y0,2)
   17 CONTINUE
      CALL DELTA(A,B,EV0,LST,LDEL,X,Y0,N,DELK,RDEL,P,Q,R,AN,BN,V,S)
      IF(DELK.GT.EPS) GO TO 7
C GO FROM MOD=2 TO MOD=0
      MOD=MODF
      PRINT 1007,EPS
      LRET=1
    3 K1=K-1
    9 IF(LDEL.EQ.2.OR.LST.EQ.4) GO TO 4
      PRINT 1002,K1,DELK,RDEL,TK,EV0
      GO TO 5
    4 PRINT 1003,K1,DELK,TK,EV0
    5 CONTINUE
      PRINT 121
      PRINT 122,(J,X(J),Y0(J),J=1,N,LX)
 121  FORMAT(3X,'J',6X,'X',11X,'Y')
 122  FORMAT(1X,I4,D12.4,D17.9)
      IF(LRET.EQ.1) RETURN
C--- RETURN WITH GIVEN PRECISION EPS
      M=M+LPR
    7 IF(K.EQ.M) GO TO 3
C ATTENSION!  GO FROM MOD=2 TO MOD=0
      IF(MOD.EQ.0) GO TO 250
      OTN=DELK2/DELK
      COTND=2. D0
      CONSTD=1.D+3
      KCON=10
      IF(OTN.GT.COTND.OR.K.LT.KCON) GO TO 250
      MOD=0
      PRINT 251
  251 FORMAT(3X,'GO FROM MOD=2 TO MOD=0')
      GO TO 16
  250 CONTINUE
      IF(MOD.EQ.0) GO TO 18
      CALL SKI(N,H,EV0,Y0,P,Q,R,S,AN,BN)
      CALL AKBK(A,B,N,H,EV0,Y0,1)
      CALL DELTA(A,B,EVF,LST,LDEL,X,Y0,N,DELV,RDELV,P,Q,R,AN,BN,V,S)
   18 CONTINUE
      CALL PROGRV(N,AN,BN,V)
      CALL SIMPYV(H,N,Y0,V,S1,S2)
      SYA2=0.D0
      SYDYA=0.D0
      IF(D2.EQ.0.) GO TO 12
      IF(LAS.EQ.0) GO TO 12
      CALL ASIM(A,B,EV0,Y0(N),SYA2,SYDYA)
   12 CONTINUE
      IF(K.EQ.1) DELK1=DELK
      IF(MOD.EQ.0) GO TO 21
      AMJU=0.5D0-0.5D0*S1-S2
      DO 19 J3=2,N1
   19 S(J3)=R(J3)*Y0(J3)
      CALL AKBK(A,B,N,H,EVF,Y0,2)
      CALL DELTA(A,B,EVF,LST,LDEL,X,Y0,N,DELV2,RDELV2,P,Q,R,AN,BN,Y,S)
      CALL PROGRV(N,AN,BN,Y)
      CALL SIMPYV(H,N,Y0,Y,S3,S4)
      AMJU=AMJU/S4
      DO 22 J=1,N
   22 V(J)=V(J)+AMJU*Y(J)
      IF(K.EQ.1)DELK1=DELK
      GO TO 23
   21 CONTINUE
      AMJU=0.5D0*(S1+1.D0-SYA2)/(S2+SYDYA)
      DO 2 J=2,N1
    2 S(J)=0.D0
   23 CONTINUE
      CALL TAUKA(LST,DELK1,DELK,T0,TK1,TK,LDEL,N,EV0,AMJU,Y0,V,Y,P,Q,R,
     ,AN,BN,A,B,S,X)
      IF(MOD.EQ.0) GO TO 25
      DO 24 I=1,N
   24 Y0(I)=Y0(I)+TK*V(I)
      CALL DOPORT(NZER,N,H,Y0,Y,V,1)
      GO TO 26
   25 DO 8 I=1,N
      Y0(I)=Y0(I)*(1.D0-TK)+TK*AMJU*V(I)
    8 CONTINUE
   26 EV0=EV0+TK*AMJU
      DELK2=DELK1
      DELK1=DELK
      TK1=TK
   10 CONTINUE
      MOD=MODF
      PRINT 1006,NT
      RETURN

 1000 FORMAT(/,1X,'RETURN FROM SLIPH4 BECAUSE N=',I6,' GT.NMAX=',I6)
 1002 FORMAT(/,1X,'ITER=',I3,' DELTA=',D9.2,' AT X=',D13.6,', TAUKA=',
     ,E16.9,/2X,' EV=',E16.9)
 1003 FORMAT(/,' ITER=',I3,' DELTA=',D9.2,' TAUKA=',D10.3,' EV=',D16.9)
 1004 FORMAT(5X,'X',9X,'Y(X)')
 1005 FORMAT(F11.4,D18.9)
 1006 FORMAT(15X,'END OF ITERATIONS. N_ITER=',I3)
 1007 FORMAT(/1X,'SOLUTION OF EIGEN VALUE PROBLEM WITH GIVEN PRECISION',
     ,' EPS=',D9.2)
 1019 FORMAT(1X,'EV0=',D16.9,' DELTA0(1) WITH PRECISION H*H=',D10.3,
     ,' AT X=',F10.5)
      END

C***********************************************************************

      SUBROUTINE DELTA(A,B,EV0,LST,LDEL,X,Y0,N,DELK,RDEL,P,Q,R,AN,BN,V,
     ,S)
      IMPLICIT REAL*8(A-H,O-Z)
C*
C ---SUBROUTINE FOR CALCULATION OF RESIDUAL OF DISCRETE SCHEME
C    OF DIFFERENTIAL EQUATION

      DIMENSION X(N),Y0(N),P(N),Q(N),R(N),AN(N),BN(N),V(N),S(N)
      COMMON /ANBN/ A23,A24,A25,B2,A34,A35,B3
      COMMON /BOUND/ D1,DD1,F1,DF1,D2,DD2,F2,DF2
      COMMON /CI1/ AM11,AM21,AM31,AM41,AM51,AM61
      COMMON /CN/ AM4,AM3,AM2,AM1,AM0,AMP1
      COMMON /CN1/ C4N,C3N,C2N,C1N,C0N,CN1
      COMMON /CMINMA/ CMIN,CMAX
      COMMON /HVAR/ H,HH,H1,H2,H3,H26,H212
      COMMON /MOD/ AK,BK,MOD

      N1=N-1
      N2=N-2
      EV=EV0
      CALL D1F1(EV,A,D1,DD1,F1,DF1)
      CALL D2F2(EV,B,D2,DD2,F2,DF2)
C--- CALCULATIONS FOR I=1
      AM11=-25.D0*D1*H1/12.D0+F1
      AM21=4.D0*D1*H1
      AM31=-3.D0*D1*H1
      AM41=AM21/3.D0
      AM51=-0.25*D1*H1
      DELK=0.D0
      IF(LDEL.EQ.2.OR.LST.EQ.4) GO TO 3
      DELK=DABS(AM11*Y0(1)+AM21*Y0(2)+AM31*Y0(3)+AM41*Y0(4)+AM51*Y0(5))
      RDEL=X(1)
    3 AM61=-AK
      AM11=AM11*HH
      AM21=AM21*HH
      AM31=AM31*HH
      AM41=AM41*HH
      AM51=AM51*HH
      AM61=AM61*HH
      DO 13 I=2,N2
      IF(I.GT.2) GO TO 6
C--- CALCULATION FOR I=2
      ALM12=-1.5D0*P(2)*H2
      ALM22=1.5D0*H1*(Q(2)-EV0*R(2))+2.D0*P(3)*H2
      ALM32=0.5D0*(3.D0*P(2)-P(4))*H2-2.D0*(Q(3)-EV0*R(3))*H1
      ALM42=0.5D0*(Q(4)-EV0*R(4))*H1-2.D0*P(3)*H2
      ALM52=0.5D0*P(4)*H2
      ALM62=0.5D0*H1*(-3.D0*S(2)+4.D0*S(3)-S(4))
      AMM12=2.D0*P(2)*H3
      AMM22=-2.D0*(Q(2)-EV0*R(2))*H2-5.D0*P(3)*H3
      AMM32=5.D0*(Q(3)-EV0*R(3))*H2+(P(5)+4.D0*P(4)-2.D0*P(2))*H3
      AMM42=H2*((5.D0*P(3)-4.D0*P(5))*H1-4.D0*(Q(4)-EV0*R(4)))
      AMM52=H2*((3.D0*P(5)-4.D0*P(4))*H1+Q(5)-EV0*R(5))
      AMM62=H2*(2.D0*S(2)-5.D0*S(3)+4.D0*S(4)-S(5))
      AMM12=H3*(P(2)+5.D0*P(3)/6.D0+2.D0*P(4)/3.D0+0.5D0*P(5))
      AMM22=-2.D0*H2*(Q(2)-EV0*R(2))+H3*(10.D0*P(2)/3.D0-
     -20.D0*P(3)/3.D0-4.D0*P(4)-8.D0*P(5)/3.D0)
      AMM32=5.D0*H2*(Q(3)-EV0*R(3))+H3*(6.D0*P(5)+12.D0*P(4)-6.D0*P(2))
      AMM42=-4.D0*H2*(Q(4)-EV0*R(4))+H3*(2.D0*P(2)+20.D0*P(3)/3.D0-
     -20.D0*P(4)/3.D0-8.D0*P(5))
      AMM52=H2*(Q(5)-EV0*R(5))+H3*(25.D0*P(5)/6.D0-2.D0*P(4)-
     -5.D0*P(3)/6.D0-P(2)/3.D0)
      AL12=-0.5D0*H1-H26*ALM12
      AL22=-H26*ALM22
      AL32=0.5D0*H1-H26*ALM32
      AL42=-H26*ALM42
      AL52=-H26*ALM52
      AL62=-H26*ALM62
      AM12=H2-H212*AMM12
      AM22=-2.D0*H2-H212*AMM22
      AM32=H2-H212*AMM32
      AM42=-H212*AMM42
      AM52=-H212*AMM52
      AM62=-H212*AMM62
      P2=2.D0*P(2)
      C12=AM12+P2*AL12
      C22=AM22+P2*AL22+Q(2)-EV0*R(2)
      C32=AM32+P2*AL32
      C42=AM42+P2*AL42
      C52=AM52+P2*AL52
      C62=AM62+P2*AL62-S(2)
      D=DABS(C12*Y0(1)+C22*Y0(2)+C32*Y0(3)+C42*Y0(4)+C52*Y0(5))
      C12=C12*HH
      C22=C22*HH
      C32=C32*HH
      C42=C42*HH
      C52=C52*HH
      C62=C62*HH
      IF(LDEL.EQ.2.OR.LST.EQ.4) GO TO 5
      IF(D.LT.DELK) GO TO 4
      DELK=D
      RDEL=X(2)
      GO TO 4
    5 DELK=DELK+D*D
    4 ZNA=C12*AM21-C22*AM11
      IF(DABS(ZNA).LT.CMIN) GO TO 50
      A23=(C12*AM31-C32*AM11)/ZNA
      A24=(C12*AM41-C42*AM11)/ZNA
      A25=(C12*AM51-C52*AM11)/ZNA
      B2=-(C12*AM61-C62*AM11)/ZNA
      GO TO 13
    6 CONTINUE
C--- CALCULATION FOR I=3,...,N-2
      QRH=(Q(I-1)-EV0*R(I-1))*H1
      QRH2=QRH*H1
      ALMI1=0.5D0*QRH-0.5D0*(P(I+1)+3.D0*P(I-1))*H2
      ALM0I=2.D0*(P(I+1)+P(I-1))*H2
      AMMI1=(3.D0*P(I-1)-2.D0*P(I)-P(I+1))*H3-QRH2
      AMMI1=AMMI1-P(I)*(QRH-H2*(P(I+1)+3.D0*P(I-1)))
      AMM0I=4.D0*(P(I+1)-P(I-1))*H3+2.D0*(Q(I)-EV0*R(I))*H2
      AMM0I=AMM0I-4.D0*P(I)*H2*(P(I+1)+P(I-1))
      QRH=(Q(I+1)-EV0*R(I+1))*H1
      QRH2=QRH*H1
      ALM1I=-0.5D0*(3.D0*P(I+1)+P(I-1))*H2-0.5D0*QRH
      ALM2I=0.5D0*H1*(S(I+1)-S(I-1))
      AMM1I=(P(I-1)+2.D0*P(I)-3.D0*P(I+1))*H3-QRH2
      AMM1I=AMM1I+P(I)*(QRH+H2*(3.D0*P(I+1)+P(I-1)))
      AMM2I=H2*(S(I+1)-2.D0*S(I)+S(I-1))
      AMM2I=AMM2I-P(I)*H1*(S(I+1)-S(I-1))
      ALI1=-0.5D0*H1-H26*ALMI1
      AL0I=-H26*ALM0I
      AL1I=0.5D0*H1-H26*ALM1I
      AL2I=-H26*ALM2I
      AMI1=H2-H212*AMMI1
      AM0I=-2.D0*H2-H212*AMM0I
      AM1I=H2-H212*AMM1I
      AM2I=-H212*AMM2I
      P2=2.D0*P(I)
      CI1=AMI1+P2*ALI1
      C0I=AM0I+P2*AL0I+Q(I)-EV0*R(I)
      C1I=AM1I+P2*AL1I
      C2I=AM2I+P2*AL2I-S(I)
      D=DABS(CI1*Y0(I-1)+C0I*Y0(I)+C1I*Y0(I+1))
      CI1=CI1*HH
      C0I=C0I*HH
      C1I=C1I*HH
      C2I=C2I*HH
      IF(LDEL.EQ.2.OR.LST.EQ.4) GO TO 7
      IF(D.LT.DELK) GO TO 8
      DELK=D
      RDEL=X(I)
      GO TO 8
    7 DELK=DELK+D*D
    8 IF(I.GT.3) GO TO 10
C--- CALCULATION FOR I=3
      ZNA=A23*CI1-C0I
      IF(DABS(ZNA).LT.CMIN) GO TO 50
      A34=(A24*CI1-C1I)/ZNA
      A35=CI1*A25/ZNA
      B3=(C2I+B2*CI1)/ZNA
      GO TO 13
   10 IF(I.GT.4) GO TO 12
C--- CALCULATION FOR I=4
      ZNA=A34*CI1-C0I
      IF(DABS(ZNA).LT.CMIN) GO TO 50
      AN(I)=(C1I-A35*CI1)/ZNA
      BN(I)=(C2I+B3*CI1)/ZNA
      GO TO 13
  12  ZNA=-AN(I-1)*CI1-C0I
C--- CALCULATION FOR I=5,6,...,N-2
      IF(DABS(ZNA).LT.CMIN) GO TO 50
      AN(I)=C1I/ZNA
      BN(I)=(C2I+BN(I-1)*CI1)/ZNA
C--- END OF CYCLE FOR DIRECT PROGONKA
   13 CONTINUE
C--- CALCULATION FOR I=N-1
      ALM4N=0.5D0*P(N-3)*H2
      ALM3N=-0.5D0*(Q(N-3)-EV0*R(N-3))*H1-2.D0*P(N-2)*H2
      ALM2N=2.D0*(Q(N-2)-EV0*R(N-2))*H1+0.5D0*(3.D0*P(N-1)-P(N-3))*H2
      ALM1N=2.D0*P(N-2)*H2-1.5D0*(Q(N-1)-EV0*R(N-1))*H1
      ALM0N=-1.5D0*P(N-1)*H2
      ALMN1=0.5D0*H1*(S(N-3)-4.D0*S(N-2)+3.D0*S(N-1))
      AMM4N=(4.D0*P(N-3)-3.D0*P(N-4))*H3+(Q(N-4)-EV0*R(N-4))*H2
      AMM3N=(4.D0*P(N-4)-5.D0*P(N-2))*H3-4.D0*(Q(N-3)-EV0*R(N-3))*H2
      AMM2N=(2.D0*P(N-1)-4.D0*P(N-3)-P(N-4))*H3+5.D0*(Q(N-2)-EV0*R(N-2))
     **H2
      AMM1N=5.D0*P(N-2)*H3-2.D0*(Q(N-1)-EV0*R(N-1))*H2
      AMM0N=-2.D0*P(N-1)*H3
      AMMN1=H2*(-S(N-4)+4.D0*S(N-3)-5.D0*S(N-2)+2.D0*S(N-1))
      AMM4N=H2*(Q(N-4)-EV0*R(N-4))+H3*(P(N-1)/3.D0+5.D0*P(N-2)/6.D0+
     +2.D0*P(N-3)-25.D0*P(N-4)/6.D0)
      AMM3N=-4.D0*H2*(Q(N-3)-EV0*R(N-3))+H3*(8.D0*P(N-4)+20.D0*P(N-3)/
     /3.D0-20.D0*P(N-2)/3.D0-2.D0*P(N-1))
      AMM2N=5.D0*H2*(Q(N-2)-EV0*R(N-2))+H3*(6.D0*P(N-1)-12.D0*P(N-3)-
     -6.D0*P(N-4))
      AMM1N=-2.D0*H2*(Q(N-1)-EV0*R(N-1))+H3*(8.D0*P(N-4)/3.D0+4.D0*
     *P(N-3)+20.D0*P(N-2)/3.D0-10.D0*P(N-1)/3.D0)
      AMM0N=-H3*(0.5D0*P(N-4)+2.D0*P(N-3)/3.D0+5.D0*P(N-2)/6.D0+P(N-1))
      AL4N=-H26*ALM4N
      AL3N=-H26*ALM3N
      AL2N=-0.5D0*H1-H26*ALM2N
      AL1N=-H26*ALM1N
      AL0N= 0.5D0*H1-H26*ALM0N
      ALN1=-H26*ALMN1
      AM4N=-H212*AMM4N
      AM3N=-H212*AMM3N
      AM2N=H2-H212*AMM2N
      AM1N=-2.D0*H2-H212*AMM1N
      AM0N=H2-H212*AMM0N
      AMN1=-H212*AMMN1
      P2=2.D0*P(N-1)
      C4N=AM4N+P2*AL4N
      C3N=AM3N+P2*AL3N
      C2N=AM2N+P2*AL2N
      C1N=AM1N+P2*AL1N+Q(N-1)-EV0*R(N-1)
      C0N=AM0N+P2*AL0N
      CN1=AMN1+P2*ALN1-S(N-1)
      D=DABS(C4N*Y0(N-4)+C3N*Y0(N-3)+C2N*Y0(N-2)+C1N*Y0(N-1)+C0N*Y0(N))
      C4N=C4N*HH
      C3N=C3N*HH
      C2N=C2N*HH
      C1N=C1N*HH
      C0N=C0N*HH
      CN1=CN1*HH
      IF(LDEL.EQ.2.OR.LST.EQ.4) GO TO 14
      IF(D.LT.DELK) GO TO 15
      DELK=D
      RDEL=X(N-1)
      GO TO 15
   14 DELK=DSQRT(H*(DELK+D*D))
   15 CONTINUE
C--- CALCULATION FOR I=N
      AM4=0.25D0*D2*H1
      AM3=-4.D0*D2*H1/3.D0
      AM2= 3.D0*D2*H1
      AM1=-4.D0*D2*H1
      AM0=25.D0*D2*H1/12.D0+F2
      AMP1=-BK
      D=DABS(AM4*Y0(N-4)+AM3*Y0(N-3)+AM2*Y0(N-2)+AM1*Y0(N-1)+AM0*Y0(N))
      AM4=AM4*HH
      AM3=AM3*HH
      AM2=AM2*HH
      AM1=AM1*HH
      AM0=AM0*HH
      AMP1=AMP1*HH
      IF(LDEL.EQ.2.OR.LST.EQ.4) RETURN
      IF(D.LT.DELK) RETURN
      DELK=D
      RDEL=X(N)
      RETURN
   50 PRINT 1021,ZNA,CMIN,I
      RETURN
 1021 FORMAT(3X,'ATTENSION! ZNA=',D16.9,' .LT. CMIN=',D9.2,' FOR I=',I4)
      END

C***********************************************************************

      SUBROUTINE DELT0(EV0,X,Y0,N,DELK,RDEL,P,Q,R)
      IMPLICIT REAL*8(A-H,O-Z)

C ---SUBROUTINE FOR CALCULATION OF RESIDUAL IN DISCRETE SCHEME
C    OF DIFFERENTIAL EQUATION    FOR Y0

      DIMENSION X(N),Y0(N),P(N),Q(N),R(N)
      COMMON /BOUND/ D1,DD1,F1,DF1,D2,DD2,F2,DF2
      COMMON /CI1/ AM11,AM21,AM31,AM41,AM51,AM61
      COMMON /CN/ AM4,AM3,AM2,AM1,AM0,AMP1
      COMMON /CN1/ C4N,C3N,C2N,C1N,C0N,CN1
      COMMON /CMINMA/ CMIN,CMAX
      COMMON /HVAR/ H,HH,H1,H2,H3,H26,H212
      COMMON /MOD/ AK,BK,MOD

      N1=N-1
      N2=N-2
      CALL D1F1(EV0,A,D1,DD1,F1,DF1)
      CALL D2F2(EV0,B,D2,DD2,F2,DF2)
C--- CALCULATION FOR I=1
      AM11=-25.D0*D1*H1/12.D0+F1
      AM21=4.D0*D1*H1
      AM31=-3.D0*D1*H1
      AM41=AM21/3.D0
      AM51=-0.25D0*D1*H1
      DELK=0.D0
      DELK=DABS(AM11*Y0(1)+AM21*Y0(2)+AM31*Y0(3)+AM41*Y0(4)+AM51*Y0(5))
      RDEL=X(1)
C--- CALCULATION FOR I=2,...,N-1
      DO 1 I=2,N1
      RD=DABS((Y0(I+1)-2.D0*Y0(I)+Y0(I-1))*H2+P(I)*(Y0(I+1)-Y0(I-1))*H1+
     +(Q(I)-EV0*R(I))*Y0(I))
      IF(DELK.GT.RD) GO TO 1
      DELK=RD
      RDEL=X(I)
    1 CONTINUE
C--- CALCULATION FOR I=N
      AM4=0.25D0*D2*H1
      AM3=-4.D0*D2*H1/3.D0
      AM2= 3.D0*D2*H1
      AM1=-4.D0*D2*H1
      AM0=25.D0*D2*H1/12.D0+F2
      D=DABS(AM4*Y0(N-4)+AM3*Y0(N-3)+AM2*Y0(N-2)+AM1*Y0(N-1)+AM0*Y0(N))
      IF(D.LT.DELK) RETURN
      DELK=D
      RDEL=X(N)
      RETURN
      END

C***********************************************************************

      SUBROUTINE AKBK(A,B,N,H,EV,Y,NV)
      IMPLICIT REAL*8(A-H,O-Z)

      DIMENSION Y(N)
      COMMON /BOUND/ D1,DD1,F1,DF1,D2,DD2,F2,DF2
      COMMON /MOD/ AK,BK,MOD

      H12=12.D0*H
      N1=N-1
      N2=N-2
      N3=N-3
      N4=N-4
      CALL D1F1(EV,A,D1,DD1,F1,DF1)
      CALL D2F2(EV,B,D2,DD2,F2,DF2)
    1 DYL=(48.D0*Y(2)-36.D0*Y(3)+16.D0*Y(4)-3.D0*Y(5))/H12
      DYR=(3.D0*Y(N4)-16.D0*Y(N3)+36.D0*Y(N2)-48.D0*Y(N1))/H12
      AK=Y(1)*(25.D0*DD1/H12-DF1)-DD1*DYL
      BK=-DD2*DYR-Y(N)*(25.D0*DD2/H12+DF2)
      IF(NV.EQ.2) RETURN
      AK=Y(1)*(25.D0* D1/H12- F1)- D1*DYL
      BK=- D2*DYR-Y(N)*(25.D0* D2/H12+ F2)
      RETURN
      END

C***********************************************************************

      SUBROUTINE SKI(N,H,EV,Y,P,Q,R,S,DY,D2Y)
      IMPLICIT REAL*8(A-H,O-Z)

      DIMENSION Y(N),P(N),Q(N),R(N),S(N),DY(N),D2Y(N)

      CALL DYH4(N,H,EV,Y,DY,P,Q,R)
      CALL D2YH4(N,H,EV,Y,D2Y,P,Q,R)
      D2Y(1)=0.D0
      D2Y(N)=0.D0
      N1=N-1
      DO 2 J=2,N1
      S(J)=-(D2Y(J)+2.D0*P(J)*DY(J)+(Q(J)-EV*R(J))*Y(J))
    2 CONTINUE
      RETURN
      END

C***********************************************************************

      SUBROUTINE DYH4(N,H,EV,Y,DY,P,Q,R)
      IMPLICIT REAL*8(A-H,O-Z)

      DIMENSION Y(N),DY(N),P(N),Q(N),R(N)

      HR=1.D0/(12.D0*H)
      H1=1.D0/H
      N1=N-1
      N2=N-2
      N3=N-3
      N4=N-4
      C16=-1.D0/6.D0
      DY(1)=HR*(-25.D0*Y(1)+48.D0*Y(2)-36.D0*Y(3)+16.D0*Y(4)-3.D0*Y(5))
      DY(N)=HR*(3.D0*Y(N4)-16.D0*Y(N3)+36.D0*Y(N2)-48.D0*Y(N1)+25.D0*
     *Y(N))
      ALM12=-1.5D0*P(2)*C16
      AL22=(1.5D0*H*(Q(2)-EV*R(2))+2.D0*P(3))*C16
      ALM32=(.5D0*(3.D0*P(2)-P(4))-2.D0*H*(Q(3)-EV*R(3)))*C16
      AL42=(0.5D0*H*(Q(4)-EV*R(4))-2.D0*P(3))*C16
      AL52=0.5D0*C16*P(4)
      AL62=0.D0
      AL12=-0.5D0*H1+ALM12
      AL32= 0.5D0*H1+ALM32
      DY(2)=AL12*Y(1)+AL22*Y(2)+AL32*Y(3)+AL42*Y(4)+AL52*Y(5)+AL62
      AL4N=-P(N3)/12.D0
      AL3N=(-0.5D0*H*(Q(N3)-EV*R(N3))-2.D0*P(N2))*C16
      ALM2N=(2.D0*H*(Q(N2)-EV*R(N2))+0.5D0*(3.D0*P(N1)-P(N3)))*C16
      AL1N=(2.D0*P(N2)-1.5D0*H*(Q(N1)-EV*R(N1)))*C16
      ALM0N=-1.5D0*C16*P(N1)
      ALN1=0.D0
      AL2N=-0.5D0*H1+ALM2N
      AL0N=0.5D0*H1+ALM0N
      DY(N1)=AL4N*Y(N4)+AL3N*Y(N3)+AL2N*Y(N2)+AL1N*Y(N1)+AL0N*Y(N)+ALN1
      DO 1 I=3,N2
      I1=I-1
      IP1=I+1
      QRH=Q(I1)-EV*R(I1)
      ALMI1=0.5D0*(H*QRH-P(IP1)-3.D0*P(I1))*C16
      AL0I=2.D0*(P(IP1)+P(I1))*C16
      ALM1I=-0.5D0*( 3.D0*P(IP1)+P(I1)+QRH*H)*C16
      AL2I=0.D0
      ALI1=-0.5D0*H1+ALMI1
      AL1I=0.5D0*H1+ALM1I
      DY(I)=ALI1*Y(I1)+AL0I*Y(I)+AL1I*Y(IP1)+AL2I
   1  CONTINUE
      RETURN
      END

C***********************************************************************

      SUBROUTINE D2YH4(N,H,EV,Y,D2Y,P,Q,R)
      IMPLICIT REAL*8(A-H,O-Z)

      DIMENSION Y(N),D2Y(N),P(N),Q(N),R(N)

      HH=H*H
      H1=1.D0/H
      H2=1.D0/HH
      N1=N-1
      N2=N-2
      N3=N-3
      N4=N-4
      C16=-1.D0/6.D0
      C112=-1.D0/12.D0
      AMM12=-H1*P(2)/6.D0
      AMM22=(-2.D0*(Q(2)-EV*R(2))-5.D0*P(3)*H1)*C112
      AMM32=(5.D0*(Q(3)-EV*R(3))+H1*(P(5)+4.D0*P(4)-2.D0*P(2)))*C112
      AM42=((5.D0*P(3)-4.D0*P(5))*H1-4.D0*(Q(4)-EV*R(4)))*C112
      AM52=((3.D0*P(5)-4.D0*P(4))*H1+Q(5)-EV*R(5))*C112
      AM62=0.D0
      AMM12=C112*H1*(P(2)+5.D0*P(3)/6.D0+2.D0*P(4)/3.D0+0.5D0*P(5))
      AMM22=(Q(2)-EV*R(2))/6.D0+H1*C112*(10.D0*P(2)/3.D0-20.D0*P(3)/
     /3.D0-4.D0*P(4)-8.D0*P(5)/3.D0)
      AMM32=(5.D0*(Q(3)-EV*R(3))+H1*(12.D0*P(4)-6.D0*P(2)+6.D0*P(5)))
     **C112
      AM42=(H1*(2.D0*P(2)+20.D0*P(3)/3.D0-20.D0*P(4)/3.D0-8.D0*P(5))-
     -4.D0*(Q(4)-EV*R(4)))*C112
      AM52=(Q(5)-EV*R(5)+H1*(25.D0*P(5)/6.D0-2.D0*P(4)-5.D0*P(3)/6.D0-
     -P(2)/3.D0))*C112
      AM12=H2+AMM12
      AM22=-2.D0*H2+AMM22
      AM32=H2+AMM32
      D2Y(2)=AM12*Y(1)+AM22*Y(2)+AM32*Y(3)+AM42*Y(4)+AM52*Y(5)+AM62
      DO 1 I=3,N2
      I1=I-1
      IP1=I+1
      QRH2=Q(I1)-EV*R(I1)
      AMMI1=(H1*(3.D0*P(I1)-2.D0*P(I)-P(IP1))-QRH2)*C112
      AMM0I=(4.D0*H1*(P(IP1)-P(I1))+2.D0*(Q(I)-EV*R(I)))*C112
      AMM1I=(H1*(P(I1) +2.D0*P(I)-3.D0*P(IP1))-(Q(IP1)-EV*R(IP1)))*C112
      AM2I=0.D0
      AMMI1=AMMI1-C112*P(I)*(H*QRH2-P(IP1)-3.D0*P(I1))
      AMM0I=AMM0I-C112*4.D0*P(I)*(P(IP1)+P(I1))
      AMM1I=AMM1I+C112*P(I)*(H*(Q(IP1)-EV*R(IP1))+3.D0*P(IP1)+P(I1))
      AMI1=H2+AMMI1
      AM0I=-2.D0*H2+AMM0I
      AM1I=H2+AMM1I
      D2Y(I)=AMI1*Y(I1)+AM0I*Y(I)+AM1I*Y(IP1)+AM2I
    1 CONTINUE
      AM4N=(H1*(4.D0*P(N3)-3.D0*P(N4))+Q(N4)-EV*R(N4))*C112
      AM3N=(H1*(4.D0*P(N4)-5.D0*P(N2))-4.D0*(Q(N3)-EV*R(N3)))*C112
      AMM2N=(H1*(2.D0*P(N1)-4.D0*P(N3)-P(N4))+5.D0*(Q(N2)-EV*R(N2)))*
     *C112
      AMM1N=(5.D0*H1*P(N2)-2.D0*(Q(N1)-EV*R(N1)))*C112
      AMM0N=-2.D0*H1*P(N1)*C112
      AMN1=0.D0
      AM4N=C112*(Q(N4)-EV*R(N4)+H1*(P(N1)/3.D0+5.D0*P(N2)/6.D0+2.D0*
     *P(N3)-25.D0*P(N4)/6.D0))
      AM3N=C112*(-4.D0*(Q(N3)-EV*R(N3))+H1*(8.D0*P(N4)+20.D0*P(N3)/3.D0-
     -20.D0*P(N2)/3.D0-2.D0*P(N1)))
      AMM2N=C112*(5.D0*(Q(N2)-EV*R(N2))+H1*(6.D0*P(N1)-12.D0*P(N3)-
     -6.D0*P(N4)))
      AMM1N=C112*(-2.D0*(Q(N1)-EV*R(N1))+H1*(8.D0*P(N4)/3.D0+4.D0*P(N3)+
     +20.D0*P(N2)/3.D0-10.D0*P(N1)/3.D0))
      AMM0N=-C112*H1*(0.5D0*P(N4)+2.D0*P(N3)/3.D0+5.D0*P(N2)/6.D0+P(N1))
      AM2N=H2+AMM2N
      AM1N=-2.D0*H2+AMM1N
      AM0N=H2+AMM0N
      D2Y(N1)=AM4N*Y(N4)+AM3N*Y(N3)+AM2N*Y(N2)+AM1N*Y(N1)+AM0N*Y(N)+AMN1
      RETURN
      END

C***********************************************************************

      SUBROUTINE PROGRV(N,AN,BN,V)
      IMPLICIT REAL*8(A-H,O-Z)
C*  SUBROUTINE FOR CALCULATION FUNCTION V(X)

      DIMENSION AN(N),BN(N),V(N)

      COMMON /ANBN/ A23,A24,A25,B2,A34,A35,B3
      COMMON /CI1/ AM11,AM21,AM31,AM41,AM51,AM61
      COMMON /CN/ AM4,AM3,AM2,AM1,AM0,AMP1
      COMMON /CN1/ C4N,C3N,C2N,C1N,C0N,CN1
      COMMON/CMINMA/ CMIN,CMAX
      DIMENSION E(5,6),INDEX(5)

      N1=N-1
      N2=N-2
      N3=N-3
      N4=N-4
      N5=N-5
      AN1N1=C4N*AN(N4)*AN(N3)*AN(N2)+C3N*AN(N3)*AN(N2)+C2N*AN(N2)+C1N
      AN1N=C0N
      BN1=-CN1-C2N*BN(N2)-C3N*(AN(N3)*BN(N2)+BN(N3))-
     -C4N*(AN(N4)*AN(N3)*BN(N2)+AN(N4)*BN(N3)+BN(N4))
      ANN1=AM4*AN(N4)*AN(N3)*AN(N2)+AM3*AN(N3)*AN(N2)+AM2*AN(N2)+AM1
      ANN=AM0
      BMN=-AMP1-AM2*BN(N2)-AM3*(AN(N3)*BN(N2)+BN(N3))-
     -AM4*(AN(N4)*AN(N3)*BN(N2)+AN(N4)*BN(N3)+BN(N4))
      ZNA=AN1N*ANN1-ANN*AN1N1
      IF(DABS(ZNA).GT.CMIN) GO TO 1
      PRINT 114
      RETURN
C--- THE REVERCE PROGONKA
    1 V(N)=(BN1*ANN1-BMN*AN1N1)/ZNA
      V(N-1)=(BMN*AN1N-BN1*ANN)/ZNA
      DO 2 I=4,N2
      J=N2-I+4
      V(J)=AN(J)*V(J+1)+BN(J)
    2 CONTINUE
      V(3)=B3-A34*V(4)-A35*V(5)
      V(2)=B2-A23*V(3)-A24*V(4)-A25*V(5)
      V(1)=-(AM21*V(2)+AM31*V(3)+AM41*V(4)+AM51*V(5)+AM61)/AM11
      DO 3 I=1,5
      DO 3 J=1,6
    3 E(I,J)=0.D0
      E(1,1)=1.D0
      E(1,2)=-AN(N4)
      E(1,6)=BN(N4)
      E(3,3)=1.D0
      E(3,4)=-AN(N2)
      E(3,6)=BN(N2)
      E(4,1)=C4N
      E(4,2)=C3N
      E(4,3)=C2N
      E(4,4)=C1N
      E(4,5)=C0N
      E(4,6)=-CN1
      E(2,2)=1.D0
      E(2,3)=-AN(N3)
      E(2,6)=BN(N3)
      E(5,1)=AM4
      E(5,2)=AM3
      E(5,3)=AM2
      E(5,4)=AM1
      E(5,5)=AM0
      E(5,6)=-AMP1
      CALL MATIN2(E,5,5,MDIM,1,INDEX,NERROR,DETERM)
      V(N)=E(5,6)
      V(N1)=E(4,6)
      V(N2)=E(3,6)
      V(N3)=E(2,6)
      V(N4)=E(1,6)
      DO 4 K=4,N5
      J=N5-K+4
    4 V(J)=AN(J)*V(J+1)+BN(J)
      V(3)=B3-A34*V(4)-A35*V(5)
      V(2)=B2-A23*V(3)-A24*V(4)-A25*V(5)
      V(1)=-(AM21*V(2)+AM31*V(3)+AM41*V(4)+AM51*V(5)+AM61)/AM11
      RETURN
  114 FORMAT(/1X,'RIGHT BOUNDARY CONDITIONS ARE UNSOLVABLE')
      END

C***********************************************************************

      SUBROUTINE SIMPYV(H,N,Y,V,S1,S2)
      IMPLICIT REAL*8(A-H,O-Z)
C*  SUBROUTINE FOR CALCULATION INTEGRALS S1,S2 FROM Y*Y AND Y*V
C*  WITH HELP OF SIMPSON'S METHOD

      DIMENSION Y(N),V(N)

      N12=(N-1)/2
      S1=Y(1)*Y(1)-Y(N)*Y(N)
      S2=Y(1)*V(1)-Y(N)*V(N)
      DO 1 J=1,N12
      I=2*J
      S1=S1+4.D0*Y(I)*Y(I)+2.D0*Y(I+1)*Y(I+1)
      S2=S2+4.D0*Y(I)*V(I)+2.D0*Y(I+1)*V(I+1)
    1 CONTINUE
      S1=H*S1/3.D0
      S2=H*S2/3.D0
      RETURN
      END

C***********************************************************************

      SUBROUTINE TAUKA(LST,DELK1,DELK,T0,TK1,TK,LDEL,N,EV0,AMJU,Y0,V,Y,
     ,P,Q,R,AN,BN,A,B,S,X)
      IMPLICIT REAL*8(A-H,O-Z)
C*  SUBROUTINE FOR CALCULATION PARAMETER TAUKA

      COMMON /MOD/ AK,BK,MOD
      DIMENSION D(10)
      DIMENSION Y0(N),V(N),Y(N),P(N),Q(N),R(N),AN(N),BN(N),S(N),X(N)

      TK=T0
      ONE=1.D0
      TWO=2.D0
      IF(LST.EQ.1) RETURN
      IF(LST.EQ.2)  GO TO 1
      IF(LST.EQ.3)  GO TO 3
      IF(LST.EQ.4)  GO TO 5
      IF(LST.EQ.5)  GO TO 7
      PRINT 100,LST,T0
      RETURN
    1 IF(DELK.GE.DELK1)GO TO 2
      TK=DMIN1(ONE,TWO*TK1)
      RETURN
    2 TK=DMAX1(T0,0.5D0*TK1)
      RETURN
    3 T=TK1*DELK1/DELK
      IF(DELK.GE.DELK1) GO TO 4
      TK=DMIN1(ONE,T)
      RETURN
    4 TK=DMAX1(T0,T)
      RETURN
    5 DO 6 I=1,N
      Y(I)=AMJU*V(I)
    6 CONTINUE
      EV1=EV0+AMJU
      CALL DELTA(A,B,EV1,LST,LDEL,X,Y,N,DEL1,RDEL,P,Q,R,AN,BN,V,S)
      TK=DELK1/(DELK1+DEL1)
      RETURN
   7  NTAU=10
      HTAU=(1.D0-T0)/(NTAU-1)
      N1=N-1
      DO 9 I=1,NTAU
      TK=T0+HTAU*(I-1)
      EV1=EV0+TK*AMJU
      IF(MOD.EQ.0) GO TO 12
      DO 8 J=1,N
      Y(J)=Y0(J)*(1.D0-TK)+TK*AMJU*V(J)
    8 CONTINUE
      GO TO 13
   12 DO 14 J=1,N
   14 Y(J)=Y0(J)+TK*V(J)
   13 N1=N-1
      DO 11 K=2,N1
   11 S(K)=0.D0
      H=(B-A)/N1
      CALL AKBK(A,B,N,H,EV1,Y,1)
      CALL DELTA(A,B,EV1,LST,LDEL,X,Y,N,D(I),RDEL,P,Q,R,AN,BN,V,S)
    9 CONTINUE
      DELK=D(1)
      DO 10 I=1,NTAU
      IF(DELK.LT.D(I)) GO TO 10
      ITAU=I
      DELK=D(I)
   10 CONTINUE
      TK=T0+(ITAU-1)*HTAU
      RETURN
  100 FORMAT(/,1X,'ATTENSION!  LST=',I4,' .GT. 5. TK=T0=',D16.9)
  101 FORMAT(3X,'FROM TAUKA: I=',I2,' THI=',D16.9,' DELTA=',D16.9)
      END

C***********************************************************************

      SUBROUTINE DOPORT(K,N,H,Y0,Y,V,ND)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y0(N),Y(N),V(N)

      REWIND ND
      IF(K.EQ.0) GO TO 6
      DO 1 J=1,N
    1 V(J)=Y0(J)
      DO 3 I=1,K
      READ(ND,100)Y
      CALL SIMPS(H,N,Y,Y0,S)
      DO 2 L=1,N
      V(L)=V(L)-S*Y(L)
    2 CONTINUE
    3 CONTINUE
      DO 5 II=1,N
      Y0(II)=V(II)
    5 CONTINUE
    6 CALL UNORM(N,H,Y0)
      WRITE(ND,100)Y0
      RETURN
  100 FORMAT(5D16.9)
      END

C***********************************************************************

      SUBROUTINE SIMPS(H,N,Y,V,S)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y(N),V(N)

      N12=(N-1)/2
      S1=Y(1)*V(1)-Y(N)*V(N)
      S=-Y(N)*V(N)
      DO 1 J=1,N12
      I1=2*J
      I=(N12-J+1)*2
      S1=S1+4.D0*Y(I1)*V(I1)+2.D0*Y(I1+1)*V(I1+1)
      S=S+4.D0*Y(I)*V(I)+2.D0*Y(I+1)*V(I+1)
   1  CONTINUE
      S=(S+Y(1)*V(1))*H/3.D0
      S1=S1*H/3.D0
      RETURN
      END

C***********************************************************************

      SUBROUTINE ASIM(A,B,EV,YN,S1,S2)
      IMPLICIT REAL*8(A-H,O-Z)
      RETURN
      END

C***********************************************************************

      SUBROUTINE  MATIN2 (A,IDIM1,N1,IDIM2,N2,INDEX,NERROR,DETERM)
C
C   MATRIX INVERSION WITH ACCOMPANYING SOLUTION OF LINEAR EQUATIONS

      DOUBLE PRECISION A,DETERM,DETER,PIVOT,SWAP
      DIMENSION A(*),INDEX(IDIM1)
      DETER=1.0D0
      N=N1
      IEMAT=N+N2
      IDIM=IDIM1
      NMIN1=N-1
C   THE ROUTINE DOES ITS OWN EVALUATION FOR DOUBLE SUBSCRIPTING OF
C   ARRAY A.
      IPIVC=1-IDIM
C   MAIN LOOP TO INVERT THE MATRIX
      DO 11 MAIN=1,N
      PIVOT=0.0D0
      IPIVC=IPIVC+IDIM
C   SEARCH FOR NEXT PIVOT IN COLUMN MAIN.
      IPIVC1=IPIVC+MAIN-1
      IPIVC2=IPIVC +NMIN1
      DO 2 I1=IPIVC1,IPIVC2
      IF(DABS(A(I1))-DABS(PIVOT)) 2,2,1
    1 PIVOT=A(I1)
      LPIV=I1
    2 CONTINUE
C   IS PIVOT DIFFERENT FROM ZERO
      IF(PIVOT) 3,15,3
C   GET THE PIVOT-LINE INDICATOR AND SWAP LINES IF NECESSARY
    3 ICOL=LPIV-IPIVC+1
      INDEX(MAIN)=ICOL
      IF(ICOL-MAIN) 6,6,4
C   COMPLEMENT THE DETERMINANT
    4 DETER=-DETER
C   POINTER TO LINE PIVOT FOUND
      ICOL=ICOL-IDIM
C   POINTER TO EXACT PIVOT LINE
      I3=MAIN-IDIM
      DO 5 I=1,IEMAT
      ICOL=ICOL+IDIM
      I3=I3+IDIM
      SWAP=A(I3)
      A(I3)=A(ICOL)
    5 A(ICOL)=SWAP
C   COMPUTE DETERMINANT
    6 DETER=DETER*PIVOT
      PIVOT=1.D0/PIVOT
C   TRANSFORM PIVOT COLUMN
      I3=IPIVC+NMIN1
      DO 7 I=IPIVC,I3
    7 A(I)=-A(I)*PIVOT
      A(IPIVC1)=PIVOT
C   PIVOT ELEMENT TRANSFORMED
C
C   NOW CONVERT REST OF THE MATRIX
      I1=MAIN-IDIM
C   POINTER TO PIVOT LINE ELEMENTS
      ICOL=1-IDIM
C   GENERAL COLUMN POINTER
      DO 10 I=1,IEMAT
      ICOL=ICOL+IDIM
      I1=I1+IDIM
C   POINTERS MOVED
      IF(I-MAIN) 8,10,8
C   PIVOT COLUMN EXCLUDED
    8 JCOL=ICOL+NMIN1
      SWAP=A(I1)
      I3=IPIVC-1
      DO 9 I2=ICOL,JCOL
      I3=I3+1
    9 A(I2)=A(I2)+SWAP*A(I3)
      A(I1)=SWAP*PIVOT
   10 CONTINUE
   11 CONTINUE
C   NOW REARRANGE THE MATRIX TO GET RIGHT INVERS
      DO 14 I1=1,N
      MAIN=N+1-I1
      LPIV=INDEX(MAIN)
      IF(LPIV-MAIN) 12,14,12
   12 ICOL=(LPIV-1)*IDIM+1
      JCOL=ICOL+NMIN1
      IPIVC=(MAIN-1)*IDIM+1-ICOL
      DO 13 I2=ICOL,JCOL
      I3=I2+IPIVC
      SWAP=A(I2)
      A(I2)=A(I3)
   13 A(I3)=SWAP
   14 CONTINUE
      DETERM=DETER
      NERROR=0
      RETURN
   15 NERROR=MAIN
      DETERM=DETER
      RETURN
      END

C********************END OF COMPLEX SLIPH4**************************
